@@ -1,22 +1,23 @@
-/* -copyright-
-#-# 
-#-# xsnow: let it snow on your desktop
-#-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020,2021,2022,2023,2024 Willem Vermin
-#-# 
-#-# This program is free software: you can redistribute it and/or modify
-#-# it under the terms of the GNU General Public License as published by
-#-# the Free Software Foundation, either version 3 of the License, or
-#-# (at your option) any later version.
-#-# 
-#-# This program is distributed in the hope that it will be useful,
-#-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-#-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#-# GNU General Public License for more details.
-#-# 
-#-# You should have received a copy of the GNU General Public License
-#-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#-# 
+/* 
+ -copyright-
+# xsnow: let it snow on your desktop
+# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
+#              2019,2020,2021,2022,2023,2024 Willem Vermin
+# 
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+# 
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# 
+#-endcopyright-
 */
 //
 #include <pthread.h>
@@ -30,6 +31,7 @@
 
 #include "xsnow-constants.h"
 
+#include "xsnow.h"
 #include "Santa.h"
 #include "birds.h"
 #include "clocks.h"
@@ -46,6 +48,7 @@
 #include "ui.h"
 #include "utils.h"
 #include "windows.h"
+#include "xpm2cairo-gdk.h"
 
 
 #define NWINGS 8
@@ -298,9 +301,7 @@ void *do_update_speed_birds(void *d)
 	 kd_free(kd);
 	 kd = kd_create(3);
 
-	 int i;
-
-	 for (i=0; i<Nbirds; i++)
+	 for (int i=0; i<Nbirds; i++)
 	 {
 	    BirdType *bird = &birds[i];
 	    kd_insert3f(kd, bird->x, bird->y, bird->z, bird);
@@ -308,7 +309,7 @@ void *do_update_speed_birds(void *d)
 
 	 int sumnum         = 0;
 	 float summeandist  = 0;
-	 for (i=0; i<Nbirds; i++)
+	 for (int i=0; i<Nbirds; i++)
 	 {
 	    if (drand48() < Flags.Anarchy*0.01)
 	       continue;
@@ -452,8 +453,7 @@ int do_update_pos_birds(void *d)
 
    P("%f\n",dt);
 
-   int i;
-   for (i=0; i<Nbirds; i++)
+   for (int i=0; i<Nbirds; i++)
    {
       BirdType *bird = &birds[i];
 
@@ -474,12 +474,9 @@ int birds_draw(cairo_t *cr)
    LEAVE_IF_INACTIVE;
    P("drawing birds %d\n",counter++);
 
-   int before;
-   int i;
-
    lock();
 
-   for (before=0; before<2; before++)
+   for (int before=0; before<2; before++)
    {
       if(before && Flags.FollowSanta && !Flags.BirdsOnly)
       {
@@ -528,9 +525,8 @@ int birds_draw(cairo_t *cr)
 	    testbird.x = attrbird.x;
 	    testbird.y = attrbird.y;
 	    testbird.z = attrbird.z;
-	    int i;
 	    int centerbird = 0;
-	    for (i=0; i<3; i++)
+	    for (int i=0; i<3; i++)
 	    {
 	       GdkPixbuf *bird_pixbuf = bird_pixbufs[testbird.wingstate+i*NWINGS];
 	       int iw = 400;
@@ -551,7 +547,7 @@ int birds_draw(cairo_t *cr)
 	 }
 #endif
       }
-      for (i=0; i<Nbirds; i++)
+      for (int i=0; i<Nbirds; i++)
       {
 	 BirdType *bird = &birds[i];
 
@@ -682,8 +678,7 @@ int birds_erase(int force)
       LEAVE_IF_INACTIVE;
 
    P("birds_erase %d\n",counter++);
-   int i;
-   for (i=0; i<Nbirds; i++)
+   for (int i=0; i<Nbirds; i++)
    {
       BirdType *bird = &birds[i];
       if (bird->prevdrawable && bird->prevw != 0 && bird->prevh != 0)
@@ -727,7 +722,6 @@ void init_birds(int start)
 {
    lock();
 
-   int i;
    if(!global.IsDouble)
       birds_erase(1);
    P("nbirds: %d %d\n",start,Flags.Nbirds);
@@ -738,7 +732,7 @@ void init_birds(int start)
       kd_free(kd);
    kd = kd_create(3);
    Nbirds = Flags.Nbirds;
-   for (i=start; i<Nbirds; i++)
+   for (int i=start; i<Nbirds; i++)
    {
       BirdType *bird = &birds[i];
       bird->x = drand48()*blobals.maxx;
@@ -788,8 +782,7 @@ static int do_wings(void *d)
 
    lock();
 
-   int i;
-   for (i=0; i<Nbirds; i++)
+   for (int i=0; i<Nbirds; i++)
    {
       BirdType *bird = &birds[i];
       bird->wingstate++;
@@ -875,8 +868,7 @@ static void main_window()
 
 void birds_init_color()
 {
-   int i;
-   for (i=0; i<NBIRDPIXBUFS; i++)
+   for (int i=0; i<NBIRDPIXBUFS; i++)
    {
       g_object_unref(bird_pixbufs[i]);
    }
@@ -886,13 +878,12 @@ void birds_init_color()
 
 static void init_bird_pixbufs(const char *color)
 {
-   int i;
-   for (i=0; i<NBIRDPIXBUFS; i++)
+   for (int i=0; i<NBIRDPIXBUFS; i++)
    {
       char **x;
       int lines;
       xpm_set_color((char **)birds_xpm[i], &x, &lines, color);
-      bird_pixbufs[i] = gdk_pixbuf_new_from_xpm_data((const char **)x);
+      bird_pixbufs[i] = xpm2gdk(global.display, (char **)x, NULL);
       xpm_destroy(x);
    }
 }
