@@ -1,22 +1,23 @@
-/* -copyright-
-#-# 
-#-# xsnow: let it snow on your desktop
-#-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020,2021,2022,2023,2024 Willem Vermin
-#-# 
-#-# This program is free software: you can redistribute it and/or modify
-#-# it under the terms of the GNU General Public License as published by
-#-# the Free Software Foundation, either version 3 of the License, or
-#-# (at your option) any later version.
-#-# 
-#-# This program is distributed in the hope that it will be useful,
-#-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-#-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#-# GNU General Public License for more details.
-#-# 
-#-# You should have received a copy of the GNU General Public License
-#-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#-# 
+/* 
+   -copyright-
+# xsnow: let it snow on your desktop
+# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
+#              2019,2020,2021,2022,2023,2024 Willem Vermin
+# 
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+# 
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# 
+#-endcopyright-
 */
 
 #include <pthread.h>
@@ -44,6 +45,7 @@
 #include "ui.h"
 #include "blowoff.h"
 #include "treesnow.h"
+#include "xpm2cairo-gdk.h"
 
 #define NOTACTIVE \
    (Flags.BirdsOnly || !WorkspaceActive() || Flags.NoSnowFlakes)
@@ -81,8 +83,6 @@ static const float  LocalScale = 0.8;
 
 void snow_init()
 {
-   int i;
-
    MaxFlakeTypes = 0;
    while(snow_xpm[MaxFlakeTypes])
       MaxFlakeTypes++;
@@ -94,10 +94,11 @@ void snow_init()
 
 
    snowPix       = (SnowMap          *)malloc(MaxFlakeTypes*sizeof(SnowMap));
+   assert(snowPix);
 
    P("MaxFlakeTypes: %d\n",MaxFlakeTypes);
 
-   for (i=0; i<MaxFlakeTypes; i++)
+   for (int i=0; i<MaxFlakeTypes; i++)
    {
       snowPix[i].surface = NULL;
    }
@@ -114,7 +115,7 @@ void snow_init()
 
    // now we would like to be able to get rid of the snow xpms:
    /*
-      for (i=0; i<MaxFlakeTypes; i++)
+      for (int i=0; i<MaxFlakeTypes; i++)
       xpm_destroy(xsnow_xpm[i]);
       free(xsnow_xpm);
       */
@@ -163,9 +164,8 @@ void snow_ui()
 void init_snow_pix()
 {
    (void)LocalScale;
-   int flake;
-   P("%d init_snow_pix\n",counter++);
-   for (flake=0; flake<MaxFlakeTypes; flake++) 
+   P("%d init_snow_pix\n",global.counter++);
+   for (int flake=0; flake<MaxFlakeTypes; flake++) 
    {
       SnowMap *rp = &snowPix[flake];
       int w,h;
@@ -185,7 +185,7 @@ void init_snow_pix()
       }
       else
 	 xpm_set_color(xsnow_xpm[flake], &x, &lines, Flags.SnowColor);
-      GdkPixbuf *pixbuf = gdk_pixbuf_new_from_xpm_data((const char **)x);
+      GdkPixbuf *pixbuf = xpm2gdk(global.display, (char **)x, NULL);
       if (w<1) w=1;
       if (h<1) h=1;
       if (w == 1 && h == 1) h = 2;
@@ -204,13 +204,13 @@ int snow_draw(cairo_t *cr)
 {
    if (Flags.NoSnowFlakes)
       return TRUE;
-   P("snow_draw %d\n",counter++);
+   P("snow_draw %d\n",global.counter++);
 
    set_begin();
    Snow *flake;
    while( (flake = (Snow *)set_next()) )
    {
-      P("snow_draw %d %f\n",counter++,ALPHA);
+      P("snow_draw %d %f\n",global.counter++,ALPHA);
       cairo_set_source_surface (cr, snowPix[flake->whatFlake].surface, flake->rx, flake->ry);
       double alpha = ALPHA;
       if (flake->fluff)
@@ -237,7 +237,7 @@ int snow_erase(int force)
       EraseSnowFlake1(flake);
       n++;
    }
-   P("snow_erase %d %d\n",counter++,n);
+   P("snow_erase %d %d\n",global.counter++,n);
    return TRUE;
 }
 
@@ -278,8 +278,7 @@ int do_genflakes(void *d)
    else
       sumdt = 0;
 
-   int i;
-   for(i=0; i<desflakes; i++)
+   for (int i=0; i<desflakes; i++)
    {
       MakeFlake(-1);
    }
@@ -297,19 +296,30 @@ int do_UpdateSnowFlake(Snow *flake)
    if ((flake->freeze || flake->fluff)  && global.RemoveFluff)
    {
       P("removefluff\n");
-      DelFlake(flake);
       EraseSnowFlake1(flake);
+      DelFlake(flake);
       return FALSE;
    }
    double FlakesDT = time_snowflakes;
    float NewX = flake->rx + (flake->vx*FlakesDT)*SnowSpeedFactor;
    float NewY = flake->ry + (flake->vy*FlakesDT)*SnowSpeedFactor;
 
+   // restore accumulation on fallensnow after some calls
+   if (flake->accum == 0)
+   {
+      const int countmax = 200;
+      flake->counter++;
+      if (flake->counter > countmax)
+      {
+	 flake->counter = 0;
+	 flake->accum   = 1;
+      }
+   }
    // handle fluff and KillFlakes
    if (KillFlakes || (flake->fluff && flake->flufftimer > flake->flufftime))
    {
-      DelFlake(flake);
       EraseSnowFlake1(flake);
+      DelFlake(flake);
       return FALSE;
    }
    if (flake->fluff)
@@ -385,7 +395,7 @@ int do_UpdateSnowFlake(Snow *flake)
    int nx = lrintf(NewX);
    int ny = lrintf(NewY);
 
-   if (!flake->fluff)
+   if (!flake->fluff && flake->accum)
    {
       Lock_fallen();
       // determine if non-fluffy-flake touches the fallen snow,
@@ -402,15 +412,15 @@ int do_UpdateSnowFlake(Snow *flake)
 	    //if(fsnow->win.id == 0 ||(fsnow->win.ws == global.CWorkSpace || fsnow->win.sticky))
 	    if(fsnow->win.id == 0 ||(IsVisibleFallen(fsnow) || fsnow->win.sticky))
 	    {
-	       if (nx >= fsnow->x && nx <= fsnow->x + fsnow->w &&
+	       //if (nx >= fsnow->x && nx <= fsnow->x + fsnow->w &&
+	       if (nx >= fsnow->x - Flags.SnowSize && nx <= fsnow->x + fsnow->w && // debug
 		     ny < fsnow->y+2)
 	       {
-		  int i;
 		  int istart = nx     - fsnow->x;
 		  int imax   = istart + flakew;
 		  if (istart < 0) istart = 0;
 		  if (imax > fsnow->w) imax = fsnow->w;
-		  for (i = istart; i < imax; i++)
+		  for (int i = istart; i < imax; i++)
 		     if (ny > fsnow->y - fsnow->acth[i] - 1)
 		     {
 			if(fsnow->acth[i] < fsnow->desh[i])
@@ -492,10 +502,9 @@ int do_UpdateSnowFlake(Snow *flake)
 	 //     move upwards until pixel is not in TreeRegion
 	 //     That pixel will be designated as snow-on-tree
 	 // Only one snow-on-tree pixel has to be found.
-	 int i;
 	 int found = 0;
 	 int xfound,yfound;
-	 for(i=0; i<flakew; i++)
+	 for (int i=0; i<flakew; i++)
 	 {
 	    if(found) break;
 	    int ybot = y+flakeh;
@@ -505,8 +514,7 @@ int do_UpdateSnowFlake(Snow *flake)
 	    if (in != CAIRO_REGION_OVERLAP_IN) // if bottom pixel not in TreeRegion, skip
 	       continue;
 	    // move upwards, until pixel is not in TreeRegion
-	    int j;
-	    for (j=ybot-1; j >= y; j--)
+	    for (int j=ybot-1; j >= y; j--)
 	    {
 	       cairo_rectangle_int_t grect = {xbot,j,1,1};
 	       cairo_region_overlap_t in = cairo_region_contains_rectangle(global.TreeRegion,&grect); 
@@ -565,6 +573,7 @@ int do_UpdateSnowFlake(Snow *flake)
 Snow *MakeFlake(int type)
 {
    Snow *flake = (Snow *)malloc(sizeof(Snow)); 
+   assert(flake);
    global.FlakeCount++; 
    if (type < 0)
    {
@@ -615,6 +624,8 @@ void InitFlake(Snow *flake)
    flake->ry         = -randint(global.SnowWinHeight/10)-flakeh;
    flake->cyclic     = 1;
    flake->fluff      = 0;
+   flake->accum      = 1;
+   flake->counter    = 0;
    flake->flufftimer = 0;
    flake->flufftime  = 0;
    flake->m          = drand48()+0.1;
@@ -693,21 +704,22 @@ void genxpmflake(char ***xpm, int w, int h)
 
    x = (float *)malloc(nmax*sizeof(float));
    y = (float *)malloc(nmax*sizeof(float));
+   assert(x);
+   assert(y);
 
-   int i,j;
    float w2 = 0.5*w;
    float h2 = 0.5*h;
 
    y[0] = 0;
    x[0] = 0;    // to have at least one pixel in the centre
    int n = 1;
-   for (i=0; i<h; i++)
+   for (int i=0; i<h; i++)
    {
       float yy = i;
       if (yy > h2)
 	 yy = h - yy;
       float py = 2*yy/h;
-      for (j=0; j<w; j++)
+      for (int j=0; j<w; j++)
       {
 	 float xx = j;
 	 if (xx > w2)
@@ -731,9 +743,11 @@ void genxpmflake(char ***xpm, int w, int h)
    float *xa, *ya;
    xa = (float *)malloc(n*sizeof(float));
    ya = (float *)malloc(n*sizeof(float));
+   assert(xa);
+   assert(ya);
 
 
-   for (i=0; i<n; i++)
+   for (int i=0; i<n; i++)
    {
       xa[i] = x[i]*cosf(a)-y[i]*sinf(a);
       ya[i] = x[i]*sinf(a)+y[i]*cosf(a);
@@ -744,7 +758,7 @@ void genxpmflake(char ***xpm, int w, int h)
    float ymin = ya[0];
    float ymax = ya[0];
 
-   for (i=0; i<n; i++)
+   for (int i=0; i<n; i++)
    {
       // smallest xa:
       if (xa[i] < xmin)
@@ -785,6 +799,7 @@ void genxpmflake(char ***xpm, int w, int h)
 
    P("allocating %d\n",(nh+3)*sizeof(char*));
    *xpm = (char **)malloc((nh+3)*sizeof(char*));
+   assert(*xpm);
    char **X = *xpm;
 
    X[0] = (char *)malloc(80*sizeof(char));
@@ -797,18 +812,21 @@ void genxpmflake(char ***xpm, int w, int h)
    int offset = 3;
    P("allocating %d\n",nw+1);
    assert(nw>=0);
-   for (i=0; i<nh; i++)
-      X[i+offset] = (char *) malloc((nw+1)*sizeof(char));
-
-   for (i=0; i<nh; i++)
+   for (int i=0; i<nh; i++)
    {
-      for(j=0; j<nw; j++)
+      X[i+offset] = (char *) malloc((nw+1)*sizeof(char));
+      assert(X[i+offset]);
+   }
+
+   for (int i=0; i<nh; i++)
+   {
+      for (int j=0; j<nw; j++)
 	 X[i+offset][j] = ' ';
       X[i+offset][nw] = 0;
    }
 
    P("max: %d %f %f %f %f\n",n,ymin,ymax,xmin,xmax);
-   for (i=0; i<n; i++)
+   for (int i=0; i<n; i++)
    {
       X[offset + (int)(ya[i]-ymin)] [(int)(xa[i]-xmin)] = c;
       P("%f %f\n",ya[i]-ymin,xa[i]-xmin);
@@ -821,11 +839,10 @@ void genxpmflake(char ***xpm, int w, int h)
 
 void add_random_flakes(int n)
 {
-   int i;
    // create a new array with snow-xpm's:
    if (xsnow_xpm)
    {
-      for (i=0; i<MaxFlakeTypes; i++)
+      for (int i=0; i<MaxFlakeTypes; i++)
 	 xpm_destroy(xsnow_xpm[i]);
       free(xsnow_xpm);
    }
@@ -835,18 +852,22 @@ void add_random_flakes(int n)
    x = (char ***)malloc((n+NFlakeTypesVintage+1)*sizeof(char **));
    int lines;
    // copy Rick's vintage flakes:
-   for (i=0; i<NFlakeTypesVintage; i++)
+   for (int i=0; i<NFlakeTypesVintage; i++)
    {
       xpm_set_color((char **)snow_xpm[i],&x[i],&lines,"snow");
       //xpm_print((char**)snow_xpm[i]);
    }
    // add n flakes:
-   for (i=0; i<n; i++)
+   for (int i=0; i<n; i++)
    {
       int w,h;
       int m = Flags.SnowSize;
+      //w = m+m*drand48();
+      //h = m+m*drand48();
+
       w = m+m*drand48();
       h = m+m*drand48();
+
       genxpmflake(&x[i+NFlakeTypesVintage],w,h);
    }
    MaxFlakeTypes   = n + NFlakeTypesVintage;
